@@ -3,6 +3,7 @@ import os, time, math
 
 from zope.interface import implements
 from twisted.internet import defer
+from foolscap.eventual import fireEventually
 import simplejson
 from allmydata.mutable.common import NotMutableError
 from allmydata.mutable.filenode import MutableFileNode
@@ -82,15 +83,41 @@ class Adder:
                 metadata = children[name][1].copy()
             else:
                 metadata = {"ctime": now,
-                            "mtime": now}
-            if new_metadata is None:
-                # update timestamps
+                            "mtime": now,
+                            "tahoe": {
+                                "linkcrtime": now,
+                                "linkmotime": now,
+                                }
+                            }
+
+            if new_metadata is not None:
+                # Overwrite all metadata.
+                newmd = new_metadata.copy()
+
+                # Except 'tahoe'.
+                if newmd.has_key('tahoe'):
+                    del newmd['tahoe']
+                if metadata.has_key('tahoe'):
+                    newmd['tahoe'] = metadata['tahoe']
+
+                metadata = newmd
+            else:
+                # For backwards compatibility with Tahoe < 1.4.0:
                 if "ctime" not in metadata:
                     metadata["ctime"] = now
                 metadata["mtime"] = now
-            else:
-                # just replace it
-                metadata = new_metadata.copy()
+
+            # update timestamps
+            sysmd = metadata.get('tahoe', {})
+            if not 'linkcrtime' in sysmd:
+                if "ctime" in metadata:
+                    # In Tahoe < 1.4.0 we used the word "ctime" to mean what Tahoe >= 1.4.0
+                    # calls "linkcrtime".
+                    sysmd["linkcrtime"] = metadata["ctime"]
+                else:
+                    sysmd["linkcrtime"] = now
+            sysmd["linkmotime"] = now
+
             children[name] = (child, metadata)
         new_contents = self.node._pack_contents(children)
         return new_contents
@@ -234,11 +261,11 @@ class NewDirectoryNode:
     def get_storage_index(self):
         return self._uri._filenode_uri.storage_index
 
-    def check(self, monitor, verify=False):
+    def check(self, monitor, verify=False, add_lease=False):
         """Perform a file check. See IChecker.check for details."""
-        return self._node.check(monitor, verify)
-    def check_and_repair(self, monitor, verify=False):
-        return self._node.check_and_repair(monitor, verify)
+        return self._node.check(monitor, verify, add_lease)
+    def check_and_repair(self, monitor, verify=False, add_lease=False):
+        return self._node.check_and_repair(monitor, verify, add_lease)
 
     def list(self):
         """I return a Deferred that fires with a dictionary mapping child
@@ -526,7 +553,7 @@ class NewDirectoryNode:
         # in the nodecache) seem to consume about 2000 bytes.
         dirkids = []
         filekids = []
-        for name, (child, metadata) in children.iteritems():
+        for name, (child, metadata) in sorted(children.iteritems()):
             verifier = child.get_verify_cap()
             # allow LIT files (for which verifier==None) to be processed
             if (verifier is not None) and (verifier in found):
@@ -537,9 +564,17 @@ class NewDirectoryNode:
                 dirkids.append( (child, childpath) )
             else:
                 filekids.append( (child, childpath) )
-        for (child, childpath) in filekids:
+        for i, (child, childpath) in enumerate(filekids):
             d.addCallback(lambda ignored, child=child, childpath=childpath:
                           walker.add_node(child, childpath))
+            # to work around the Deferred tail-recursion problem
+            # (specifically the defer.succeed flavor) requires us to avoid
+            # doing more than 158 LIT files in a row. We insert a turn break
+            # once every 100 files (LIT or CHK) to preserve some stack space
+            # for other code. This is a different expression of the same
+            # Twisted problem as in #237.
+            if i % 100 == 99:
+                d.addCallback(lambda ignored: fireEventually())
         for (child, childpath) in dirkids:
             d.addCallback(lambda ignored, child=child, childpath=childpath:
                           self._deep_traverse_dirnode(child, childpath,
@@ -560,11 +595,11 @@ class NewDirectoryNode:
         # children for which we've got both a write-cap and a read-cap
         return self.deep_traverse(DeepStats(self))
 
-    def start_deep_check(self, verify=False):
-        return self.deep_traverse(DeepChecker(self, verify, repair=False))
+    def start_deep_check(self, verify=False, add_lease=False):
+        return self.deep_traverse(DeepChecker(self, verify, repair=False, add_lease=add_lease))
 
-    def start_deep_check_and_repair(self, verify=False):
-        return self.deep_traverse(DeepChecker(self, verify, repair=True))
+    def start_deep_check_and_repair(self, verify=False, add_lease=False):
+        return self.deep_traverse(DeepChecker(self, verify, repair=True, add_lease=add_lease))
 
 
 
@@ -695,13 +730,14 @@ class ManifestWalker(DeepStats):
 
 
 class DeepChecker:
-    def __init__(self, root, verify, repair):
+    def __init__(self, root, verify, repair, add_lease):
         root_si = root.get_storage_index()
         self._lp = log.msg(format="deep-check starting (%(si)s),"
                            " verify=%(verify)s, repair=%(repair)s",
                            si=base32.b2a(root_si), verify=verify, repair=repair)
         self._verify = verify
         self._repair = repair
+        self._add_lease = add_lease
         if repair:
             self._results = DeepCheckAndRepairResults(root_si)
         else:
@@ -714,10 +750,10 @@ class DeepChecker:
 
     def add_node(self, node, childpath):
         if self._repair:
-            d = node.check_and_repair(self.monitor, self._verify)
+            d = node.check_and_repair(self.monitor, self._verify, self._add_lease)
             d.addCallback(self._results.add_check_and_repair, childpath)
         else:
-            d = node.check(self.monitor, self._verify)
+            d = node.check(self.monitor, self._verify, self._add_lease)
             d.addCallback(self._results.add_check, childpath)
         d.addCallback(lambda ignored: self._stats.add_node(node, childpath))
         return d
