@@ -7,7 +7,8 @@ from twisted.internet import defer
 from twisted.python import failure
 from allmydata.interfaces import IPublishStatus, FileTooLargeError
 from allmydata.util import base32, hashutil, mathutil, idlib, log
-from allmydata import hashtree, codec, storage
+from allmydata import hashtree, codec
+from allmydata.storage.server import si_b2a
 from pycryptopp.cipher.aes import AES
 from foolscap.eventual import eventually
 
@@ -78,6 +79,9 @@ class PublishStatus:
     def set_active(self, value):
         self.active = value
 
+class LoopLimitExceededError(Exception):
+    pass
+
 class Publish:
     """I represent a single act of publishing the mutable file to the grid. I
     will only publish my data if the servermap I am using still represents
@@ -100,7 +104,7 @@ class Publish:
         self._node = filenode
         self._servermap = servermap
         self._storage_index = self._node.get_storage_index()
-        self._log_prefix = prefix = storage.si_b2a(self._storage_index)[:5]
+        self._log_prefix = prefix = si_b2a(self._storage_index)[:5]
         num = self._node._client.log("Publish(%s): starting" % prefix)
         self._log_number = num
         self._running = True
@@ -289,7 +293,7 @@ class Publish:
 
         self.looplimit -= 1
         if self.looplimit <= 0:
-            raise RuntimeError("loop limit exceeded")
+            raise LoopLimitExceededError("loop limit exceeded")
 
         if self.surprised:
             # don't send out any new shares, just wait for the outstanding
