@@ -6,8 +6,8 @@ from twisted.internet import defer
 from twisted.python import failure
 from foolscap import DeadReferenceError
 from foolscap.eventual import eventually
-from allmydata.util import base32, hashutil, idlib, log
-from allmydata import storage
+from allmydata.util import base32, hashutil, idlib, log, rrefutil
+from allmydata.storage.server import si_b2a
 from allmydata.interfaces import IServermapUpdaterStatus
 from pycryptopp.publickey import rsa
 
@@ -338,7 +338,8 @@ class ServerMap:
 
 
 class ServermapUpdater:
-    def __init__(self, filenode, monitor, servermap, mode=MODE_READ):
+    def __init__(self, filenode, monitor, servermap, mode=MODE_READ,
+                 add_lease=False):
         """I update a servermap, locating a sufficient number of useful
         shares and remembering where they are located.
 
@@ -348,6 +349,7 @@ class ServermapUpdater:
         self._monitor = monitor
         self._servermap = servermap
         self.mode = mode
+        self._add_lease = add_lease
         self._running = True
 
         self._storage_index = filenode.get_storage_index()
@@ -372,7 +374,7 @@ class ServermapUpdater:
         # fixed-size slots so we can retrieve less data. For now, we'll just
         # read 2000 bytes, which also happens to read enough actual data to
         # pre-fetch a 9-entry dirnode.
-        self._read_size = 2000
+        self._read_size = 4000
         if mode == MODE_CHECK:
             # we use unpack_prefix_and_signature, so we need 1k
             self._read_size = 1000
@@ -383,7 +385,7 @@ class ServermapUpdater:
         # to ask for it during the check, we'll have problems doing the
         # publish.
 
-        prefix = storage.si_b2a(self._storage_index)[:5]
+        prefix = si_b2a(self._storage_index)[:5]
         self._log_number = log.msg(format="SharemapUpdater(%(si)s): starting (%(mode)s)",
                                    si=prefix, mode=mode)
 
@@ -536,6 +538,24 @@ class ServermapUpdater:
 
     def _do_read(self, ss, peerid, storage_index, shnums, readv):
         d = ss.callRemote("slot_readv", storage_index, shnums, readv)
+        if self._add_lease:
+            renew_secret = self._node.get_renewal_secret(peerid)
+            cancel_secret = self._node.get_cancel_secret(peerid)
+            d2 = ss.callRemote("add_lease", storage_index,
+                               renew_secret, cancel_secret)
+            dl = defer.DeferredList([d, d2], consumeErrors=True)
+            def _done(res):
+                [(readv_success, readv_result),
+                 (addlease_success, addlease_result)] = res
+                if (not addlease_success and
+                    not rrefutil.check_remote(addlease_result, IndexError)):
+                    # tahoe 1.3.0 raised IndexError on non-existant buckets,
+                    # which we ignore. Unfortunately tahoe <1.3.0 had a bug
+                    # and raised KeyError, which we report.
+                    return addlease_result # propagate error
+                return readv_result
+            dl.addCallback(_done)
+            return dl
         return d
 
     def _got_results(self, datavs, peerid, readsize, stuff, started):
