@@ -1,27 +1,23 @@
 
-import datetime, os.path, re, types
+import datetime, os.path, re, types, ConfigParser, tempfile
 from base64 import b32decode, b32encode
 
-from twisted.python import log as tahoe_log
+from twisted.python import log as twlog
 from twisted.application import service
 from twisted.internet import defer, reactor
 from foolscap import Tub, eventual
 import foolscap.logging.log
-from allmydata import get_package_versions_string
+from allmydata import get_package_versions, get_package_versions_string
 from allmydata.util import log
-from allmydata.util import fileutil, iputil, observer, humanreadable
-from allmydata.util.assertutil import precondition
+from allmydata.util import fileutil, iputil, observer
+from allmydata.util.assertutil import precondition, _assert
 
-# Just to get their versions:
-import allmydata, pycryptopp, zfec
+from foolscap.logging import app_versions
 
-from foolscap.logging.publish import LogPublisher
 # Add our application versions to the data that Foolscap's LogPublisher
-# reports. Our __version__ attributes are actually instances of a "Version"
-# class, so convert them into strings first.
-LogPublisher.versions['allmydata'] = str(allmydata.__version__)
-LogPublisher.versions['zfec'] = str(zfec.__version__)
-LogPublisher.versions['pycryptopp'] = str(pycryptopp.__version__)
+# reports.
+for thing, things_version in get_package_versions().iteritems():
+    app_versions.add_version(thing, str(things_version))
 
 # group 1 will be addr (dotted quad string), group 3 if any will be portnum (string)
 ADDR_RE=re.compile("^([1-9][0-9]*\.[1-9][0-9]*\.[1-9][0-9]*\.[1-9][0-9]*)(:([1-9][0-9]*))?$")
@@ -42,55 +38,146 @@ such as private keys.  On Unix-like systems, the permissions on this directory
 are set to disallow users other than its owner from reading the contents of
 the files.   See the 'configuration.txt' documentation file for details."""
 
+class _None: # used as a marker in get_config()
+    pass
+
+class MissingConfigEntry(Exception):
+    pass
+
 class Node(service.MultiService):
     # this implements common functionality of both Client nodes and Introducer
     # nodes.
     NODETYPE = "unknown NODETYPE"
     PORTNUMFILE = None
     CERTFILE = "node.pem"
-    LOCAL_IP_FILE = "advertised_ip_addresses"
 
     def __init__(self, basedir="."):
         service.MultiService.__init__(self)
         self.basedir = os.path.abspath(basedir)
+        self._portnumfile = os.path.join(self.basedir, self.PORTNUMFILE)
         self._tub_ready_observerlist = observer.OneShotObserverList()
         fileutil.make_dirs(os.path.join(self.basedir, "private"), 0700)
         open(os.path.join(self.basedir, "private", "README"), "w").write(PRIV_README)
-        certfile = os.path.join(self.basedir, "private", self.CERTFILE)
-        self.tub = Tub(certFile=certfile)
-        self.tub.setOption("logLocalFailures", True)
-        self.tub.setOption("logRemoteFailures", True)
-        self.nodeid = b32decode(self.tub.tubID.upper()) # binary format
-        self.write_config("my_nodeid", b32encode(self.nodeid).lower() + "\n")
-        self.short_nodeid = b32encode(self.nodeid).lower()[:8] # ready for printing
-        assert self.PORTNUMFILE, "Your node.Node subclass must provide PORTNUMFILE"
-        self._portnumfile = os.path.join(self.basedir, self.PORTNUMFILE)
-        try:
-            portnum = int(open(self._portnumfile, "rU").read())
-        except (EnvironmentError, ValueError):
-            portnum = 0
-        self.tub.listenOn("tcp:%d" % portnum)
-        # we must wait until our service has started before we can find out
-        # our IP address and thus do tub.setLocation, and we can't register
-        # any services with the Tub until after that point
-        self.tub.setServiceParent(self)
+
+        # creates self.config, populates from distinct files if necessary
+        self.read_config()
+
+        nickname_utf8 = self.get_config("node", "nickname", "<unspecified>")
+        self.nickname = nickname_utf8.decode("utf-8")
+
+        self.init_tempdir()
+        self.create_tub()
         self.logSource="Node"
 
-        AUTHKEYSFILEBASE = "authorized_keys."
-        for f in os.listdir(self.basedir):
-            if f.startswith(AUTHKEYSFILEBASE):
-                keyfile = os.path.join(self.basedir, f)
-                portnum = int(f[len(AUTHKEYSFILEBASE):])
-                from allmydata import manhole
-                m = manhole.AuthorizedKeysManhole(portnum, keyfile)
-                m.setServiceParent(self)
-                self.log("AuthorizedKeysManhole listening on %d" % portnum)
-
+        self.setup_ssh()
         self.setup_logging()
         self.log("Node constructed. " + get_package_versions_string())
         iputil.increase_rlimits()
 
-    def get_config(self, name, required=False):
+    def init_tempdir(self):
+        local_tempdir = "tmp" # default is NODEDIR/tmp/
+        tempdir = self.get_config("node", "tempdir", local_tempdir)
+        tempdir = os.path.join(self.basedir, tempdir)
+        if not os.path.exists(tempdir):
+            fileutil.make_dirs(tempdir)
+        tempfile.tempdir = os.path.abspath(tempdir)
+        # this should cause twisted.web.http (which uses
+        # tempfile.TemporaryFile) to put large request bodies in the given
+        # directory. Without this, the default temp dir is usually /tmp/,
+        # which is frequently too small.
+        test_name = tempfile.mktemp()
+        _assert(os.path.dirname(test_name) == tempdir, test_name, tempdir)
+
+    def get_config(self, section, option, default=_None, boolean=False):
+        try:
+            if boolean:
+                return self.config.getboolean(section, option)
+            return self.config.get(section, option)
+        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
+            if default is _None:
+                fn = os.path.join(self.basedir, "tahoe.cfg")
+                raise MissingConfigEntry("%s is missing the [%s]%s entry"
+                                         % (fn, section, option))
+            return default
+
+    def set_config(self, section, option, value):
+        if not self.config.has_section(section):
+            self.config.add_section(section)
+        self.config.set(section, option, value)
+        assert self.config.get(section, option) == value
+
+    def read_config(self):
+        self.config = ConfigParser.SafeConfigParser()
+        self.config.read([os.path.join(self.basedir, "tahoe.cfg")])
+        self.read_old_config_files()
+
+    def read_old_config_files(self):
+        # backwards-compatibility: individual files will override the
+        # contents of tahoe.cfg
+        copy = self._copy_config_from_file
+
+        copy("nickname", "node", "nickname")
+        copy("webport", "node", "web.port")
+
+        cfg_tubport = self.get_config("node", "tub.port", "")
+        if not cfg_tubport:
+            # For 'tub.port', tahoe.cfg overrides the individual file on
+            # disk. So only read self._portnumfile is tahoe.cfg doesn't
+            # provide a value.
+            try:
+                file_tubport = open(self._portnumfile, "rU").read().strip()
+                self.set_config("node", "tub.port", file_tubport)
+            except EnvironmentError:
+                pass
+
+        copy("keepalive_timeout", "node", "timeout.keepalive")
+        copy("disconnect_timeout", "node", "timeout.disconnect")
+
+    def _copy_config_from_file(self, config_filename, section, keyname):
+        s = self.get_config_from_file(config_filename)
+        if s is not None:
+            self.set_config(section, keyname, s)
+
+    def create_tub(self):
+        certfile = os.path.join(self.basedir, "private", self.CERTFILE)
+        self.tub = Tub(certFile=certfile)
+        self.tub.setOption("logLocalFailures", True)
+        self.tub.setOption("logRemoteFailures", True)
+
+        # see #521 for a discussion of how to pick these timeout values.
+        keepalive_timeout_s = self.get_config("node", "timeout.keepalive", "")
+        if keepalive_timeout_s:
+            self.tub.setOption("keepaliveTimeout", int(keepalive_timeout_s))
+        disconnect_timeout_s = self.get_config("node", "timeout.disconnect", "")
+        if disconnect_timeout_s:
+            # N.B.: this is in seconds, so use "1800" to get 30min
+            self.tub.setOption("disconnectTimeout", int(disconnect_timeout_s))
+
+        self.nodeid = b32decode(self.tub.tubID.upper()) # binary format
+        self.write_config("my_nodeid", b32encode(self.nodeid).lower() + "\n")
+        self.short_nodeid = b32encode(self.nodeid).lower()[:8] # ready for printing
+
+        tubport = self.get_config("node", "tub.port", "tcp:0")
+        self.tub.listenOn(tubport)
+        # we must wait until our service has started before we can find out
+        # our IP address and thus do tub.setLocation, and we can't register
+        # any services with the Tub until after that point
+        self.tub.setServiceParent(self)
+
+    def setup_ssh(self):
+        ssh_port = self.get_config("node", "ssh.port", "")
+        if ssh_port:
+            ssh_keyfile = self.get_config("node", "ssh.authorized_keys_file")
+            from allmydata import manhole
+            m = manhole.AuthorizedKeysManhole(ssh_port, ssh_keyfile)
+            m.setServiceParent(self)
+            self.log("AuthorizedKeysManhole listening on %s" % ssh_port)
+
+    def get_app_versions(self):
+        # TODO: merge this with allmydata.get_package_versions
+        return dict(app_versions.versions)
+
+    def get_config_from_file(self, name, required=False):
         """Get the (string) contents of a config file, or None if the file
         did not exist. If required=True, raise an exception rather than
         returning None. Any leading or trailing whitespace will be stripped
@@ -124,7 +211,7 @@ class Node(service.MultiService):
         which is expected to return a string.
         """
         privname = os.path.join("private", name)
-        value = self.get_config(privname)
+        value = self.get_config_from_file(privname)
         if value is None:
             if isinstance(default, (str, unicode)):
                 value = default
@@ -175,12 +262,12 @@ class Node(service.MultiService):
 
     def _service_startup_failed(self, failure):
         self.log('_startService() failed')
-        tahoe_log.err(failure)
+        log.err(failure)
         print "Node._startService failed, aborting"
         print failure
         #reactor.stop() # for unknown reasons, reactor.stop() isn't working.  [ ] TODO
         self.log('calling os.abort()')
-        tahoe_log.msg('calling os.abort()')
+        twlog.msg('calling os.abort()') # make sure it gets into twistd.log
         print "calling os.abort()"
         os.abort()
 
@@ -202,17 +289,21 @@ class Node(service.MultiService):
     def setup_logging(self):
         # we replace the formatTime() method of the log observer that twistd
         # set up for us, with a method that uses better timestamps.
-        for o in tahoe_log.theLogPublisher.observers:
+        for o in twlog.theLogPublisher.observers:
             # o might be a FileLogObserver's .emit method
             if type(o) is type(self.setup_logging): # bound method
                 ob = o.im_self
-                if isinstance(ob, tahoe_log.FileLogObserver):
+                if isinstance(ob, twlog.FileLogObserver):
                     newmeth = types.UnboundMethodType(formatTimeTahoeStyle, ob, ob.__class__)
                     ob.formatTime = newmeth
         # TODO: twisted >2.5.0 offers maxRotatedFiles=50
 
         self.tub.setOption("logport-furlfile",
                            os.path.join(self.basedir, "private","logport.furl"))
+        lgfurl = self.get_config("node", "log_gatherer.furl", "")
+        if lgfurl:
+            # this is in addition to the contents of log-gatherer-furlfile
+            self.tub.setOption("log-gatherer-furl", lgfurl)
         self.tub.setOption("log-gatherer-furlfile",
                            os.path.join(self.basedir, "log_gatherer.furl"))
         self.tub.setOption("bridge-twisted-logs", True)
@@ -223,47 +314,21 @@ class Node(service.MultiService):
     def log(self, *args, **kwargs):
         return log.msg(*args, **kwargs)
 
-    def old_log(self, msg, src="", args=(), **kw):
-        if src:
-            logsrc = src
-        else:
-            logsrc = self.logSource
-        if args:
-            try:
-                msg = msg % tuple(map(humanreadable.hr, args))
-            except TypeError, e:
-                msg = "ERROR: output string '%s' contained invalid %% expansion, error: %s, args: %s\n" % (`msg`, e, `args`)
-        msg = self.short_nodeid + ": " + humanreadable.hr(msg)
-        return tahoe_log.callWithContext({"system":logsrc},
-                                         tahoe_log.msg, msg, **kw)
-
     def _setup_tub(self, local_addresses):
         # we can't get a dynamically-assigned portnum until our Tub is
         # running, which means after startService.
         l = self.tub.getListeners()[0]
         portnum = l.getPortnum()
-        # record which port we're listening on, so we can grab the same one next time
+        # record which port we're listening on, so we can grab the same one
+        # next time
         open(self._portnumfile, "w").write("%d\n" % portnum)
 
-        local_addresses = [ "%s:%d" % (addr, portnum,) for addr in local_addresses ]
-
-        addresses = []
-        try:
-            for addrline in open(os.path.join(self.basedir, self.LOCAL_IP_FILE), "rU"):
-                mo = ADDR_RE.search(addrline)
-                if mo:
-                    (addr, dummy, aportnum,) = mo.groups()
-                    if aportnum is None:
-                        aportnum = portnum
-                    addresses.append("%s:%d" % (addr, int(aportnum),))
-        except EnvironmentError:
-            pass
-
-        addresses.extend(local_addresses)
-
-        location = ",".join(addresses)
+        base_location = ",".join([ "%s:%d" % (addr, portnum)
+                                   for addr in local_addresses ])
+        location = self.get_config("node", "tub.location", base_location)
         self.log("Tub location set to %s" % location)
         self.tub.setLocation(location)
+
         return self.tub
 
     def when_tub_ready(self):
