@@ -707,6 +707,14 @@ class ProvisioningTool(rend.Page):
               all_sections,
               ]
 
+        try:
+            from allmydata import reliability
+            # we import this just to test to see if the page is available
+            _hush_pyflakes = reliability
+            f = [T.div[T.a(href="reliability.html")["Reliability Math"]], f]
+        except ImportError:
+            pass
+
         return f
 
     def file_availability(self, k, n, server_dBA):
