@@ -1,5 +1,4 @@
-
-import os.path, re, sys
+import os.path, re, sys, fnmatch
 from twisted.python import usage
 from allmydata.scripts.common import BaseOptions, get_aliases
 
@@ -58,12 +57,22 @@ class AddAliasOptions(VDriveOptions):
         self.alias = alias
         self.cap = cap
 
+    def getSynopsis(self):
+        return "%s add-alias ALIAS DIRCAP" % (os.path.basename(sys.argv[0]),)
+
+    longdesc = """Add a new alias for an existing directory."""
+
 class CreateAliasOptions(VDriveOptions):
     def parseArgs(self, alias):
         self.alias = alias
 
+    def getSynopsis(self):
+        return "%s create-alias ALIAS" % (os.path.basename(sys.argv[0]),)
+
+    longdesc = """Creates a new directory and adds an alias for it."""
+
 class ListAliasOptions(VDriveOptions):
-    pass
+    longdesc = """Displays a table of all configured aliases."""
 
 class ListOptions(VDriveOptions):
     optFlags = [
@@ -160,6 +169,9 @@ class CpOptions(VDriveOptions):
     optFlags = [
         ("recursive", "r", "Copy source directory recursively."),
         ("verbose", "v", "Be noisy about what is happening."),
+        ("caps-only", None,
+         "When copying to local files, write out filecaps instead of actual "
+         "data. (only useful for debugging and tree-comparison purposes)"),
         ]
     def parseArgs(self, *args):
         if len(args) < 2:
@@ -172,7 +184,7 @@ class RmOptions(VDriveOptions):
         self.where = where
 
     def getSynopsis(self):
-        return "%s rm VE_FILE" % (os.path.basename(sys.argv[0]),)
+        return "%s rm VDRIVE_FILE" % (os.path.basename(sys.argv[0]),)
 
 class MvOptions(VDriveOptions):
     def parseArgs(self, frompath, topath):
@@ -190,6 +202,9 @@ class LnOptions(VDriveOptions):
     def getSynopsis(self):
         return "%s ln FROM TO" % (os.path.basename(sys.argv[0]),)
 
+class BackupConfigurationError(Exception):
+    pass
+
 class BackupOptions(VDriveOptions):
     optFlags = [
         ("verbose", "v", "Be noisy about what is happening."),
@@ -197,6 +212,14 @@ class BackupOptions(VDriveOptions):
         ("ignore-timestamps", None, "Do not use backupdb timestamps to decide if a local file is unchanged."),
         ]
 
+    vcs_patterns = ('CVS', 'RCS', 'SCCS', '.git', '.gitignore', '.cvsignore', '.svn',
+                   '.arch-ids','{arch}', '=RELEASE-ID', '=meta-update', '=update',
+                   '.bzr', '.bzrignore', '.bzrtags', '.hg', '.hgignore', '_darcs')
+
+    def __init__(self):
+        super(BackupOptions, self).__init__()
+        self['exclude'] = set()
+
     def parseArgs(self, localdir, topath):
         self.from_dir = localdir
         self.to_dir = topath
@@ -204,6 +227,45 @@ class BackupOptions(VDriveOptions):
     def getSynopsis(Self):
         return "%s backup FROM ALIAS:TO" % os.path.basename(sys.argv[0])
 
+    def opt_exclude(self, pattern):
+        """Ignore files matching a glob pattern. You may give multiple
+        '--exclude' options."""
+        g = pattern.strip()
+        if g:
+            exclude = self['exclude']
+            exclude.add(g)
+
+    def opt_exclude_from(self, filepath):
+        """Ignore file matching glob patterns listed in file, one per
+        line."""
+        try:
+            exclude_file = file(filepath)
+        except:
+            raise BackupConfigurationError('Error opening exclude file %r.' % filepath)
+        try:
+            for line in exclude_file:
+                self.opt_exclude(line)
+        finally:
+            exclude_file.close()
+
+    def opt_exclude_vcs(self):
+        """Exclude files and directories used by following version
+        control systems: 'CVS', 'RCS', 'SCCS', 'SVN', 'Arch',
+        'Bazaar', 'Mercurial', and 'Darcs'."""
+        for pattern in self.vcs_patterns:
+            self.opt_exclude(pattern)
+
+    def filter_listdir(self, listdir):
+        """Yields non-excluded childpaths in path."""
+        exclude = self['exclude']
+        exclude_regexps = [re.compile(fnmatch.translate(pat)) for pat in exclude]
+        for filename in listdir:
+            for regexp in exclude_regexps:
+                if regexp.match(filename):
+                    break
+            else:
+                yield filename
+
     longdesc = """Add a versioned backup of the local FROM directory to a timestamped subdir of the (tahoe) TO/Archives directory, sharing as many files and directories as possible with the previous backup. Creates TO/Latest as a reference to the latest backup. Behaves somewhat like 'rsync -a --link-dest=TO/Archives/(previous) FROM TO/Archives/(new); ln -sf TO/Archives/(new) TO/Latest'."""
 
 class WebopenOptions(VDriveOptions):
@@ -244,9 +306,10 @@ class StatsOptions(VDriveOptions):
 
 class CheckOptions(VDriveOptions):
     optFlags = [
-        ("raw", "r", "Display raw JSON data instead of parsed"),
-        ("verify", "v", "Verify all hashes, instead of merely querying share presence"),
-        ("repair", "r", "Automatically repair any problems found"),
+        ("raw", None, "Display raw JSON data instead of parsed"),
+        ("verify", None, "Verify all hashes, instead of merely querying share presence"),
+        ("repair", None, "Automatically repair any problems found"),
+        ("add-lease", None, "Add/renew lease on all shares"),
         ]
     def parseArgs(self, where=''):
         self.where = where
@@ -258,9 +321,11 @@ class CheckOptions(VDriveOptions):
 
 class DeepCheckOptions(VDriveOptions):
     optFlags = [
-        ("raw", "r", "Display raw JSON data instead of parsed"),
-        ("verify", "v", "Verify all hashes, instead of merely querying share presence"),
-        ("repair", "r", "Automatically repair any problems found"),
+        ("raw", None, "Display raw JSON data instead of parsed"),
+        ("verify", None, "Verify all hashes, instead of merely querying share presence"),
+        ("repair", None, "Automatically repair any problems found"),
+        ("add-lease", None, "Add/renew lease on all shares"),
+        ("verbose", "v", "Be noisy about what is happening."),
         ]
     def parseArgs(self, where=''):
         self.where = where
@@ -403,4 +468,3 @@ dispatch = {
     "check": check,
     "deep-check": deepcheck,
     }
-
