@@ -13,7 +13,9 @@ from allmydata.interfaces import IURI, IMutableFileNode, IFileNode, \
 from allmydata.check_results import CheckResults, CheckAndRepairResults, \
      DeepCheckResults, DeepCheckAndRepairResults
 from allmydata.mutable.common import CorruptShareError
-from allmydata.storage import storage_index_to_dir
+from allmydata.mutable.layout import unpack_header
+from allmydata.storage.server import storage_index_to_dir
+from allmydata.storage.mutable import MutableShareFile
 from allmydata.util import hashutil, log, fileutil, pollmixin
 from allmydata.util.assertutil import precondition
 from allmydata.stats import StatsGathererService
@@ -53,7 +55,7 @@ class FakeCHKFileNode:
     def get_storage_index(self):
         return self.storage_index
 
-    def check(self, monitor, verify=False):
+    def check(self, monitor, verify=False, add_lease=False):
         r = CheckResults(self.my_uri, self.storage_index)
         is_bad = self.bad_shares.get(self.storage_index, None)
         data = {}
@@ -81,7 +83,7 @@ class FakeCHKFileNode:
         r.set_data(data)
         r.set_needs_rebalancing(False)
         return defer.succeed(r)
-    def check_and_repair(self, monitor, verify=False):
+    def check_and_repair(self, monitor, verify=False, add_lease=False):
         d = self.check(verify)
         def _got(cr):
             r = CheckAndRepairResults(self.storage_index)
@@ -97,7 +99,7 @@ class FakeCHKFileNode:
 
     def download(self, target):
         if self.my_uri.to_string() not in self.all_contents:
-            f = failure.Failure(NotEnoughSharesError())
+            f = failure.Failure(NotEnoughSharesError(None, 0, 3))
             target.fail(f)
             return defer.fail(f)
         data = self.all_contents[self.my_uri.to_string()]
@@ -107,14 +109,14 @@ class FakeCHKFileNode:
         return defer.maybeDeferred(target.finish)
     def download_to_data(self):
         if self.my_uri.to_string() not in self.all_contents:
-            return defer.fail(NotEnoughSharesError())
+            return defer.fail(NotEnoughSharesError(None, 0, 3))
         data = self.all_contents[self.my_uri.to_string()]
         return defer.succeed(data)
     def get_size(self):
         try:
             data = self.all_contents[self.my_uri.to_string()]
         except KeyError, le:
-            raise NotEnoughSharesError(le)
+            raise NotEnoughSharesError(le, 0, 3)
         return len(data)
     def read(self, consumer, offset=0, size=None):
         d = self.download_to_data()
@@ -189,7 +191,7 @@ class FakeMutableFileNode:
     def get_storage_index(self):
         return self.storage_index
 
-    def check(self, monitor, verify=False):
+    def check(self, monitor, verify=False, add_lease=False):
         r = CheckResults(self.my_uri, self.storage_index)
         is_bad = self.bad_shares.get(self.storage_index, None)
         data = {}
@@ -219,7 +221,7 @@ class FakeMutableFileNode:
         r.set_needs_rebalancing(False)
         return defer.succeed(r)
 
-    def check_and_repair(self, monitor, verify=False):
+    def check_and_repair(self, monitor, verify=False, add_lease=False):
         d = self.check(verify)
         def _got(cr):
             r = CheckAndRepairResults(self.storage_index)
@@ -228,7 +230,7 @@ class FakeMutableFileNode:
         d.addCallback(_got)
         return d
 
-    def deep_check(self, verify=False):
+    def deep_check(self, verify=False, add_lease=False):
         d = self.check(verify)
         def _done(r):
             dr = DeepCheckResults(self.storage_index)
@@ -237,7 +239,7 @@ class FakeMutableFileNode:
         d.addCallback(_done)
         return d
 
-    def deep_check_and_repair(self, verify=False):
+    def deep_check_and_repair(self, verify=False, add_lease=False):
         d = self.check_and_repair(verify)
         def _done(r):
             dr = DeepCheckAndRepairResults(self.storage_index)
@@ -267,7 +269,7 @@ class FakeMutableFileNode:
 
     def download(self, target):
         if self.storage_index not in self.all_contents:
-            f = failure.Failure(NotEnoughSharesError())
+            f = failure.Failure(NotEnoughSharesError(None, 0, 3))
             target.fail(f)
             return defer.fail(f)
         data = self.all_contents[self.storage_index]
@@ -277,7 +279,7 @@ class FakeMutableFileNode:
         return defer.maybeDeferred(target.finish)
     def download_to_data(self):
         if self.storage_index not in self.all_contents:
-            return defer.fail(NotEnoughSharesError())
+            return defer.fail(NotEnoughSharesError(None, 0, 3))
         data = self.all_contents[self.storage_index]
         return defer.succeed(data)
 
@@ -1072,6 +1074,38 @@ class WebErrorMixin:
         f.trap(WebError)
         print "Web Error:", f.value, ":", f.value.response
         return f
+
+    def _shouldHTTPError(self, res, which, validator):
+        if isinstance(res, failure.Failure):
+            res.trap(WebError)
+            return validator(res)
+        else:
+            self.fail("%s was supposed to Error, not get '%s'" % (which, res))
+
+    def shouldHTTPError(self, which,
+                        code=None, substring=None, response_substring=None,
+                        callable=None, *args, **kwargs):
+        # returns a Deferred with the response body
+        assert substring is None or isinstance(substring, str)
+        assert callable
+        def _validate(f):
+            if code is not None:
+                self.failUnlessEqual(f.value.status, str(code))
+            if substring:
+                code_string = str(f)
+                self.failUnless(substring in code_string,
+                                "substring '%s' not in '%s'"
+                                % (substring, code_string))
+            response_body = f.value.response
+            if response_substring:
+                self.failUnless(response_substring in response_body,
+                                "response substring '%s' not in '%s'"
+                                % (response_substring, response_body))
+            return response_body
+        d = defer.maybeDeferred(callable, *args, **kwargs)
+        d.addBoth(self._shouldHTTPError, which, _validate)
+        return d
+
 class ErrorMixin(WebErrorMixin):
     def explain_error(self, f):
         if f.check(defer.FirstError):
@@ -1255,11 +1289,24 @@ def _corrupt_offset_of_uri_extension_to_force_short_read(data, debug=False):
             log.msg("testing: corrupting offset %d, size %d, changing %d to %d (len(data) == %d)" % (0x48, 8, struct.unpack(">Q", data[0x48:0x48+8])[0], len(data)-0x0c-3, len(data)))
         return data[:0x48] + struct.pack(">Q", len(data)-0x0c-3) + data[0x48+8:]
 
+def _corrupt_mutable_share_data(data):
+    prefix = data[:32]
+    assert prefix == MutableShareFile.MAGIC, "This function is designed to corrupt mutable shares of v1, and the magic number doesn't look right: %r vs %r" % (prefix, MutableShareFile.MAGIC)
+    data_offset = MutableShareFile.DATA_OFFSET
+    sharetype = data[data_offset:data_offset+1]
+    assert sharetype == "\x00", "non-SDMF mutable shares not supported"
+    (version, ig_seqnum, ig_roothash, ig_IV, ig_k, ig_N, ig_segsize,
+     ig_datalen, offsets) = unpack_header(data[data_offset:])
+    assert version == 0, "this function only handles v0 SDMF files"
+    start = data_offset + offsets["share_data"]
+    length = data_offset + offsets["enc_privkey"] - start
+    return corrupt_field(data, start, length)
+
 def _corrupt_share_data(data):
     """ Scramble the file data -- the field containing the share data itself will have one
     bit flipped or else will be changed to a random value. """
     sharevernum = struct.unpack(">L", data[0x0c:0x0c+4])[0]
-    assert sharevernum in (1, 2), "This test is designed to corrupt immutable shares of v1 or v2 in specific ways."
+    assert sharevernum in (1, 2), "This test is designed to corrupt immutable shares of v1 or v2 in specific ways, not v%d." % sharevernum
     if sharevernum == 1:
         sharedatasize = struct.unpack(">L", data[0x0c+0x08:0x0c+0x08+4])[0]
 
