@@ -7,8 +7,9 @@ from twisted.python import log
 import allmydata
 from allmydata import client
 from allmydata.introducer.client import IntroducerClient
-from allmydata.util import base32, testutil
+from allmydata.util import base32
 from foolscap.eventual import flushEventualQueue
+import common_util as testutil
 
 class FakeIntroducerClient(IntroducerClient):
     def __init__(self):
@@ -33,6 +34,33 @@ class Basic(unittest.TestCase):
         open(os.path.join(basedir, "introducer.furl"), "w").write("")
         c = client.Client(basedir)
 
+    def test_loadable_old_config_bits(self):
+        basedir = "test_client.Basic.test_loadable_old_config_bits"
+        os.mkdir(basedir)
+        open(os.path.join(basedir, "introducer.furl"), "w").write("")
+        open(os.path.join(basedir, "vdrive.furl"), "w").write("")
+        open(os.path.join(basedir, "no_storage"), "w").write("")
+        open(os.path.join(basedir, "readonly_storage"), "w").write("")
+        open(os.path.join(basedir, "debug_discard_storage"), "w").write("")
+        c = client.Client(basedir)
+        try:
+            c.getServiceNamed("storage")
+            self.fail("that was supposed to fail")
+        except KeyError:
+            pass
+
+    def test_loadable_old_storage_config_bits(self):
+        basedir = "test_client.Basic.test_loadable_old_storage_config_bits"
+        os.mkdir(basedir)
+        open(os.path.join(basedir, "introducer.furl"), "w").write("")
+        open(os.path.join(basedir, "vdrive.furl"), "w").write("")
+        open(os.path.join(basedir, "readonly_storage"), "w").write("")
+        open(os.path.join(basedir, "debug_discard_storage"), "w").write("")
+        c = client.Client(basedir)
+        s = c.getServiceNamed("storage")
+        self.failUnless(s.no_storage)
+        self.failUnless(s.readonly_storage)
+
     def test_secrets(self):
         basedir = "test_client.Basic.test_secrets"
         os.mkdir(basedir)
@@ -46,52 +74,71 @@ class Basic(unittest.TestCase):
         cancel_secret = c.get_cancel_secret()
         self.failUnless(base32.b2a(cancel_secret))
 
-    def test_sizelimit_1(self):
-        basedir = "client.Basic.test_sizelimit_1"
-        os.mkdir(basedir)
-        open(os.path.join(basedir, "introducer.furl"), "w").write("")
-        open(os.path.join(basedir, "vdrive.furl"), "w").write("")
-        open(os.path.join(basedir, "sizelimit"), "w").write("1000")
-        c = client.Client(basedir)
-        self.failUnlessEqual(c.getServiceNamed("storage").sizelimit, 1000)
+    BASECONFIG = ("[client]\n"
+                  "introducer.furl = \n"
+                  )
 
-    def test_sizelimit_2(self):
-        basedir = "client.Basic.test_sizelimit_2"
+    def test_reserved_1(self):
+        basedir = "client.Basic.test_reserved_1"
         os.mkdir(basedir)
-        open(os.path.join(basedir, "introducer.furl"), "w").write("")
-        open(os.path.join(basedir, "vdrive.furl"), "w").write("")
-        open(os.path.join(basedir, "sizelimit"), "w").write("10K")
+        f = open(os.path.join(basedir, "tahoe.cfg"), "w")
+        f.write(self.BASECONFIG)
+        f.write("[storage]\n")
+        f.write("enabled = true\n")
+        f.write("reserved_space = 1000\n")
+        f.close()
         c = client.Client(basedir)
-        self.failUnlessEqual(c.getServiceNamed("storage").sizelimit, 10*1000)
+        self.failUnlessEqual(c.getServiceNamed("storage").reserved_space, 1000)
 
-    def test_sizelimit_3(self):
-        basedir = "client.Basic.test_sizelimit_3"
+    def test_reserved_2(self):
+        basedir = "client.Basic.test_reserved_2"
         os.mkdir(basedir)
-        open(os.path.join(basedir, "introducer.furl"), "w").write("")
-        open(os.path.join(basedir, "vdrive.furl"), "w").write("")
-        open(os.path.join(basedir, "sizelimit"), "w").write("5mB")
+        f = open(os.path.join(basedir, "tahoe.cfg"), "w")
+        f.write(self.BASECONFIG)
+        f.write("[storage]\n")
+        f.write("enabled = true\n")
+        f.write("reserved_space = 10K\n")
+        f.close()
         c = client.Client(basedir)
-        self.failUnlessEqual(c.getServiceNamed("storage").sizelimit,
+        self.failUnlessEqual(c.getServiceNamed("storage").reserved_space, 10*1000)
+
+    def test_reserved_3(self):
+        basedir = "client.Basic.test_reserved_3"
+        os.mkdir(basedir)
+        f = open(os.path.join(basedir, "tahoe.cfg"), "w")
+        f.write(self.BASECONFIG)
+        f.write("[storage]\n")
+        f.write("enabled = true\n")
+        f.write("reserved_space = 5mB\n")
+        f.close()
+        c = client.Client(basedir)
+        self.failUnlessEqual(c.getServiceNamed("storage").reserved_space,
                              5*1000*1000)
 
-    def test_sizelimit_4(self):
-        basedir = "client.Basic.test_sizelimit_4"
+    def test_reserved_4(self):
+        basedir = "client.Basic.test_reserved_4"
         os.mkdir(basedir)
-        open(os.path.join(basedir, "introducer.furl"), "w").write("")
-        open(os.path.join(basedir, "vdrive.furl"), "w").write("")
-        open(os.path.join(basedir, "sizelimit"), "w").write("78Gb")
+        f = open(os.path.join(basedir, "tahoe.cfg"), "w")
+        f.write(self.BASECONFIG)
+        f.write("[storage]\n")
+        f.write("enabled = true\n")
+        f.write("reserved_space = 78Gb\n")
+        f.close()
         c = client.Client(basedir)
-        self.failUnlessEqual(c.getServiceNamed("storage").sizelimit,
+        self.failUnlessEqual(c.getServiceNamed("storage").reserved_space,
                              78*1000*1000*1000)
 
-    def test_sizelimit_bad(self):
-        basedir = "client.Basic.test_sizelimit_bad"
+    def test_reserved_bad(self):
+        basedir = "client.Basic.test_reserved_bad"
         os.mkdir(basedir)
-        open(os.path.join(basedir, "introducer.furl"), "w").write("")
-        open(os.path.join(basedir, "vdrive.furl"), "w").write("")
-        open(os.path.join(basedir, "sizelimit"), "w").write("bogus")
+        f = open(os.path.join(basedir, "tahoe.cfg"), "w")
+        f.write(self.BASECONFIG)
+        f.write("[storage]\n")
+        f.write("enabled = true\n")
+        f.write("reserved_space = bogus\n")
+        f.close()
         c = client.Client(basedir)
-        self.failUnlessEqual(c.getServiceNamed("storage").sizelimit, None)
+        self.failUnlessEqual(c.getServiceNamed("storage").reserved_space, 0)
 
     def _permute(self, c, key):
         return [ peerid
@@ -125,13 +172,14 @@ class Basic(unittest.TestCase):
         open(os.path.join(basedir, "vdrive.furl"), "w").write("")
         c = client.Client(basedir)
         ss = c.getServiceNamed("storage")
-        mine, oldest = ss.remote_get_versions()
-        self.failUnlessEqual(mine, str(allmydata.__version__))
+        verdict = ss.remote_get_version()
+        self.failUnlessEqual(verdict["application-version"],
+                             str(allmydata.__full_version__))
         self.failIfEqual(str(allmydata.__version__), "unknown")
-        self.failUnless("." in str(allmydata.__version__),
+        self.failUnless("." in str(allmydata.__full_version__),
                         "non-numeric version in '%s'" % allmydata.__version__)
         all_versions = allmydata.get_package_versions_string()
-        self.failUnless("allmydata" in all_versions)
+        self.failUnless("allmydata-tahoe" in all_versions)
         log.msg("tahoe versions: %s" % all_versions)
         # also test stats
         stats = c.get_stats()
