@@ -14,7 +14,8 @@ from allmydata.mutable.common import UncoordinatedWriteError
 from allmydata.util import hashutil, base32
 from allmydata.monitor import Monitor
 from allmydata.test.common import make_chk_file_uri, make_mutable_file_uri, \
-     FakeDirectoryNode, create_chk_filenode, ErrorMixin, SystemTestMixin
+     FakeDirectoryNode, create_chk_filenode, ErrorMixin
+from allmydata.test.no_network import GridTestMixin
 from allmydata.check_results import CheckResults, CheckAndRepairResults
 import common_util as testutil
 
@@ -41,13 +42,13 @@ class Marker:
     def get_storage_index(self):
         return self.storage_index
 
-    def check(self, monitor, verify=False):
+    def check(self, monitor, verify=False, add_lease=False):
         r = CheckResults(uri.from_string(self.nodeuri), None)
         r.set_healthy(True)
         r.set_recoverable(True)
         return defer.succeed(r)
 
-    def check_and_repair(self, monitor, verify=False):
+    def check_and_repair(self, monitor, verify=False, add_lease=False):
         d = self.check(verify)
         def _got(cr):
             r = CheckAndRepairResults(None)
@@ -103,38 +104,6 @@ class Dirnode(unittest.TestCase,
         d.addCallback(_done)
         return d
 
-    def test_corrupt(self):
-        d = self.client.create_empty_dirnode()
-        def _created(dn):
-            u = make_mutable_file_uri()
-            d = dn.set_uri(u"child", u.to_string(), {})
-            d.addCallback(lambda res: dn.list())
-            def _check1(children):
-                self.failUnless(u"child" in children)
-            d.addCallback(_check1)
-            d.addCallback(lambda res:
-                          self.shouldFail(NoSuchChildError, "get bogus", None,
-                                          dn.get, u"bogus"))
-            def _corrupt(res):
-                filenode = dn._node
-                si = IURI(filenode.get_uri()).storage_index
-                old_contents = filenode.all_contents[si]
-                # We happen to know that the writecap MAC is near the end of the string. Flip
-                # one of its bits and make sure we ignore the corruption.
-                new_contents = testutil.flip_bit(old_contents, -10)
-                # TODO: also test flipping bits in the other portions
-                filenode.all_contents[si] = new_contents
-            d.addCallback(_corrupt)
-            def _check2(res):
-                d = dn.list()
-                def _c3(res):
-                    self.failUnless(res.has_key('child'))
-                d.addCallback(_c3)
-            d.addCallback(_check2)
-            return d
-        d.addCallback(_created)
-        return d
-
     def test_check(self):
         d = self.client.create_empty_dirnode()
         d.addCallback(lambda dn: dn.check(Monitor()))
@@ -414,8 +383,8 @@ class Dirnode(unittest.TestCase,
 
             d.addCallback(lambda res: n.get_metadata_for(u"child"))
             d.addCallback(lambda metadata:
-                          self.failUnlessEqual(sorted(metadata.keys()),
-                                               ["ctime", "mtime"]))
+                          self.failUnlessEqual(set(metadata.keys()),
+                                               set(["tahoe", "ctime", "mtime"])))
 
             d.addCallback(lambda res:
                           self.shouldFail(NoSuchChildError, "gcamap-no",
@@ -437,8 +406,8 @@ class Dirnode(unittest.TestCase,
                 child, metadata = res
                 self.failUnlessEqual(child.get_uri(),
                                      fake_file_uri.to_string())
-                self.failUnlessEqual(sorted(metadata.keys()),
-                                     ["ctime", "mtime"])
+                self.failUnlessEqual(set(metadata.keys()),
+                                     set(["tahoe", "ctime", "mtime"]))
             d.addCallback(_check_child_and_metadata2)
 
             d.addCallback(lambda res:
@@ -446,22 +415,31 @@ class Dirnode(unittest.TestCase,
             def _check_child_and_metadata3(res):
                 child, metadata = res
                 self.failUnless(isinstance(child, FakeDirectoryNode))
-                self.failUnlessEqual(sorted(metadata.keys()),
-                                     ["ctime", "mtime"])
+                self.failUnlessEqual(set(metadata.keys()),
+                                     set(["tahoe", "ctime", "mtime"]))
             d.addCallback(_check_child_and_metadata3)
 
             # set_uri + metadata
             # it should be possible to add a child without any metadata
             d.addCallback(lambda res: n.set_uri(u"c2", fake_file_uri.to_string(), {}))
             d.addCallback(lambda res: n.get_metadata_for(u"c2"))
-            d.addCallback(lambda metadata: self.failUnlessEqual(metadata, {}))
+            d.addCallback(lambda metadata: self.failUnlessEqual(metadata.keys(), ['tahoe']))
+
+            # You can't override the link timestamps.
+            d.addCallback(lambda res: n.set_uri(u"c2", fake_file_uri.to_string(), { 'tahoe': {'linkcrtime': "bogus"}}))
+            d.addCallback(lambda res: n.get_metadata_for(u"c2"))
+            def _has_good_linkcrtime(metadata):
+                self.failUnless(metadata.has_key('tahoe'))
+                self.failUnless(metadata['tahoe'].has_key('linkcrtime'))
+                self.failIfEqual(metadata['tahoe']['linkcrtime'], 'bogus')
+            d.addCallback(_has_good_linkcrtime)
 
             # if we don't set any defaults, the child should get timestamps
             d.addCallback(lambda res: n.set_uri(u"c3", fake_file_uri.to_string()))
             d.addCallback(lambda res: n.get_metadata_for(u"c3"))
             d.addCallback(lambda metadata:
-                          self.failUnlessEqual(sorted(metadata.keys()),
-                                               ["ctime", "mtime"]))
+                          self.failUnlessEqual(set(metadata.keys()),
+                                               set(["tahoe", "ctime", "mtime"])))
 
             # or we can add specific metadata at set_uri() time, which
             # overrides the timestamps
@@ -469,7 +447,8 @@ class Dirnode(unittest.TestCase,
                                                 {"key": "value"}))
             d.addCallback(lambda res: n.get_metadata_for(u"c4"))
             d.addCallback(lambda metadata:
-                          self.failUnlessEqual(metadata, {"key": "value"}))
+                              self.failUnless((set(metadata.keys()) == set(["key", "tahoe"])) and
+                                              (metadata['key'] == "value"), metadata))
 
             d.addCallback(lambda res: n.delete(u"c2"))
             d.addCallback(lambda res: n.delete(u"c3"))
@@ -485,14 +464,14 @@ class Dirnode(unittest.TestCase,
                                           n.set_node, u"d2", n2,
                                           overwrite=False))
             d.addCallback(lambda res: n.get_metadata_for(u"d2"))
-            d.addCallback(lambda metadata: self.failUnlessEqual(metadata, {}))
+            d.addCallback(lambda metadata: self.failUnlessEqual(metadata.keys(), ['tahoe']))
 
             # if we don't set any defaults, the child should get timestamps
             d.addCallback(lambda res: n.set_node(u"d3", n))
             d.addCallback(lambda res: n.get_metadata_for(u"d3"))
             d.addCallback(lambda metadata:
-                          self.failUnlessEqual(sorted(metadata.keys()),
-                                               ["ctime", "mtime"]))
+                          self.failUnlessEqual(set(metadata.keys()),
+                                               set(["tahoe", "ctime", "mtime"])))
 
             # or we can add specific metadata at set_node() time, which
             # overrides the timestamps
@@ -500,7 +479,8 @@ class Dirnode(unittest.TestCase,
                                                 {"key": "value"}))
             d.addCallback(lambda res: n.get_metadata_for(u"d4"))
             d.addCallback(lambda metadata:
-                          self.failUnlessEqual(metadata, {"key": "value"}))
+                          self.failUnless((set(metadata.keys()) == set(["key", "tahoe"])) and
+                                          (metadata['key'] == "value"), metadata))
 
             d.addCallback(lambda res: n.delete(u"d2"))
             d.addCallback(lambda res: n.delete(u"d3"))
@@ -524,13 +504,15 @@ class Dirnode(unittest.TestCase,
             d.addCallback(lambda children: self.failIf(u"new" in children))
             d.addCallback(lambda res: n.get_metadata_for(u"e1"))
             d.addCallback(lambda metadata:
-                          self.failUnlessEqual(sorted(metadata.keys()),
-                                               ["ctime", "mtime"]))
+                          self.failUnlessEqual(set(metadata.keys()),
+                                               set(["tahoe", "ctime", "mtime"])))
             d.addCallback(lambda res: n.get_metadata_for(u"e2"))
-            d.addCallback(lambda metadata: self.failUnlessEqual(metadata, {}))
+            d.addCallback(lambda metadata:
+                          self.failUnlessEqual(set(metadata.keys()), set(['tahoe'])))
             d.addCallback(lambda res: n.get_metadata_for(u"e3"))
             d.addCallback(lambda metadata:
-                          self.failUnlessEqual(metadata, {"key": "value"}))
+                              self.failUnless((set(metadata.keys()) == set(["key", "tahoe"]))
+                                              and (metadata['key'] == "value"), metadata))
 
             d.addCallback(lambda res: n.delete(u"e1"))
             d.addCallback(lambda res: n.delete(u"e2"))
@@ -554,13 +536,15 @@ class Dirnode(unittest.TestCase,
             d.addCallback(lambda children: self.failIf(u"new" in children))
             d.addCallback(lambda res: n.get_metadata_for(u"f1"))
             d.addCallback(lambda metadata:
-                          self.failUnlessEqual(sorted(metadata.keys()),
-                                               ["ctime", "mtime"]))
+                          self.failUnlessEqual(set(metadata.keys()),
+                                               set(["tahoe", "ctime", "mtime"])))
             d.addCallback(lambda res: n.get_metadata_for(u"f2"))
-            d.addCallback(lambda metadata: self.failUnlessEqual(metadata, {}))
+            d.addCallback(
+                lambda metadata: self.failUnlessEqual(set(metadata.keys()), set(['tahoe'])))
             d.addCallback(lambda res: n.get_metadata_for(u"f3"))
             d.addCallback(lambda metadata:
-                          self.failUnlessEqual(metadata, {"key": "value"}))
+                              self.failUnless((set(metadata.keys()) == set(["key", "tahoe"])) and
+                                              (metadata['key'] == "value"), metadata))
 
             d.addCallback(lambda res: n.delete(u"f1"))
             d.addCallback(lambda res: n.delete(u"f2"))
@@ -626,8 +610,8 @@ class Dirnode(unittest.TestCase,
             d.addCallback(lambda res: n.set_node(u"no_timestamps", n))
             d.addCallback(lambda res: n.get_metadata_for(u"no_timestamps"))
             d.addCallback(lambda metadata:
-                          self.failUnlessEqual(sorted(metadata.keys()),
-                                               ["ctime", "mtime"]))
+                          self.failUnlessEqual(set(metadata.keys()),
+                                               set(["tahoe", "ctime", "mtime"])))
             d.addCallback(lambda res: n.delete(u"no_timestamps"))
 
             d.addCallback(lambda res: n.delete(u"subdir"))
@@ -657,8 +641,8 @@ class Dirnode(unittest.TestCase,
                                                sorted([u"child", u"newfile"])))
             d.addCallback(lambda res: n.get_metadata_for(u"newfile"))
             d.addCallback(lambda metadata:
-                          self.failUnlessEqual(sorted(metadata.keys()),
-                                               ["ctime", "mtime"]))
+                          self.failUnlessEqual(set(metadata.keys()),
+                                               set(["tahoe", "ctime", "mtime"])))
 
             d.addCallback(lambda res: n.add_file(u"newfile-metadata",
                                                  uploadable,
@@ -667,7 +651,8 @@ class Dirnode(unittest.TestCase,
                           self.failUnless(IFileNode.providedBy(newnode)))
             d.addCallback(lambda res: n.get_metadata_for(u"newfile-metadata"))
             d.addCallback(lambda metadata:
-                          self.failUnlessEqual(metadata, {"key": "value"}))
+                              self.failUnless((set(metadata.keys()) == set(["key", "tahoe"])) and
+                                              (metadata['key'] == "value"), metadata))
             d.addCallback(lambda res: n.delete(u"newfile-metadata"))
 
             d.addCallback(lambda res: n.create_empty_directory(u"subdir2"))
@@ -786,7 +771,7 @@ class UCWEingNewDirectoryNode(dirnode.NewDirectoryNode):
     filenode_class = UCWEingMutableFileNode
 
 
-class Deleter(SystemTestMixin, unittest.TestCase):
+class Deleter(GridTestMixin, unittest.TestCase):
     def test_retry(self):
         # ticket #550, a dirnode.delete which experiences an
         # UncoordinatedWriteError will fail with an incorrect "you're
@@ -799,8 +784,9 @@ class Deleter(SystemTestMixin, unittest.TestCase):
         # succeed.
 
         self.basedir = self.mktemp()
-        d = self.set_up_nodes()
-        d.addCallback(lambda ignored: self.clients[0].create_empty_dirnode())
+        self.set_up_grid()
+        c0 = self.g.clients[0]
+        d = c0.create_empty_dirnode()
         small = upload.Data("Small enough for a LIT", None)
         def _created_dir(dn):
             self.root = dn
@@ -808,7 +794,7 @@ class Deleter(SystemTestMixin, unittest.TestCase):
             return dn.add_file(u"file", small)
         d.addCallback(_created_dir)
         def _do_delete(ignored):
-            n = UCWEingNewDirectoryNode(self.clients[0]).init_from_uri(self.root_uri)
+            n = UCWEingNewDirectoryNode(c0).init_from_uri(self.root_uri)
             assert n._node.please_ucwe_after_next_upload == False
             n._node.please_ucwe_after_next_upload = True
             # This should succeed, not raise an exception
