@@ -9,12 +9,14 @@ from twisted.python import log
 from foolscap import Tub, Referenceable
 from foolscap.eventual import fireEventually, flushEventualQueue
 from twisted.application import service
+from allmydata.interfaces import InsufficientVersionError
 from allmydata.introducer.client import IntroducerClient
 from allmydata.introducer.server import IntroducerService
 # test compatibility with old introducer .tac files
 from allmydata.introducer import IntroducerNode
 from allmydata.introducer import old
-from allmydata.util import testutil, idlib
+from allmydata.util import idlib, pollmixin
+import common_util as testutil
 
 class FakeNode(Referenceable):
     pass
@@ -46,7 +48,7 @@ class ServiceMixin:
         d.addCallback(flushEventualQueue)
         return d
 
-class Introducer(ServiceMixin, unittest.TestCase, testutil.PollMixin):
+class Introducer(ServiceMixin, unittest.TestCase, pollmixin.PollMixin):
 
     def test_create(self):
         ic = IntroducerClient(None, "introducer.furl", "my_nickname",
@@ -75,7 +77,7 @@ class Introducer(ServiceMixin, unittest.TestCase, testutil.PollMixin):
         self.failUnlessEqual(len(i.get_announcements()), 2)
         self.failUnlessEqual(len(i.get_subscribers()), 0)
 
-class SystemTestMixin(ServiceMixin, testutil.PollMixin):
+class SystemTestMixin(ServiceMixin, pollmixin.PollMixin):
 
     def setUp(self):
         ServiceMixin.setUp(self)
@@ -152,6 +154,9 @@ class SystemTest(SystemTestMixin, unittest.TestCase):
                 self.failUnlessEqual(len(c.get_all_peerids()), NUMCLIENTS)
                 self.failUnlessEqual(len(c.get_all_connections_for("storage")),
                                      NUMCLIENTS)
+                nodeid0 = b32decode(tubs[clients[0]].tubID.upper())
+                self.failUnlessEqual(c.get_nickname_for_peerid(nodeid0),
+                                     "nickname-0")
         d.addCallback(_check1)
 
         origin_c = clients[0]
@@ -226,3 +231,44 @@ class SystemTest(SystemTestMixin, unittest.TestCase):
                 self.failIf(c.connected_to_introducer())
         d.addCallback(_check4)
         return d
+
+class TooNewServer(IntroducerService):
+    VERSION = { "http://allmydata.org/tahoe/protocols/introducer/v999":
+                 { },
+                "application-version": "greetings from the crazy future",
+                }
+
+class NonV1Server(SystemTestMixin, unittest.TestCase):
+    # if the 1.3.0 client connects to a server that doesn't provide the 'v1'
+    # protocol, it is supposed to provide a useful error instead of a weird
+    # exception.
+
+    def test_failure(self):
+        i = TooNewServer()
+        i.setServiceParent(self.parent)
+        self.introducer_furl = self.central_tub.registerReference(i)
+
+        tub = Tub()
+        tub.setServiceParent(self.parent)
+        l = tub.listenOn("tcp:0")
+        portnum = l.getPortnum()
+        tub.setLocation("localhost:%d" % portnum)
+
+        n = FakeNode()
+        c = IntroducerClient(tub, self.introducer_furl,
+                             "nickname-client", "version", "oldest")
+        c.subscribe_to("storage")
+
+        c.setServiceParent(self.parent)
+
+        # now we wait for it to connect and notice the bad version
+
+        def _got_bad():
+            return bool(c._introducer_error) or bool(c._publisher)
+        d = self.poll(_got_bad)
+        def _done(res):
+            self.failUnless(c._introducer_error)
+            self.failUnless(c._introducer_error.check(InsufficientVersionError))
+        d.addCallback(_done)
+        return d
+
