@@ -1,36 +1,32 @@
-
 from base64 import b32encode
 import os, sys, time, re, simplejson
 from cStringIO import StringIO
+from zope.interface import implements
 from twisted.trial import unittest
 from twisted.internet import defer
 from twisted.internet import threads # CLI tests use deferToThread
 from twisted.internet.error import ConnectionDone, ConnectionLost
-from twisted.application import service
+from twisted.internet.interfaces import IConsumer, IPushProducer
 import allmydata
-from allmydata import client, uri, storage, offloaded
-from allmydata.immutable import download, upload, filenode
-from allmydata.introducer.server import IntroducerNode
-from allmydata.util import fileutil, idlib, mathutil, testutil
+from allmydata import uri
+from allmydata.storage.mutable import MutableShareFile
+from allmydata.storage.server import si_a2b
+from allmydata.immutable import download, filenode, offloaded, upload
+from allmydata.util import idlib, mathutil
 from allmydata.util import log, base32
-from allmydata.scripts import runner, cli
-from allmydata.interfaces import IDirectoryNode, IFileNode, IFileURI
+from allmydata.scripts import runner
+from allmydata.interfaces import IDirectoryNode, IFileNode, IFileURI, \
+     NoSuchChildError, NotEnoughSharesError
+from allmydata.monitor import Monitor
 from allmydata.mutable.common import NotMutableError
 from allmydata.mutable import layout as mutable_layout
-from allmydata.stats import PickleStatsGatherer
-from allmydata.key_generator import KeyGeneratorService
-from foolscap.eventual import flushEventualQueue, fireEventually
-from foolscap import DeadReferenceError, Tub
+from foolscap import DeadReferenceError
 from twisted.python.failure import Failure
 from twisted.web.client import getPage
 from twisted.web.error import Error
 
-def flush_but_dont_ignore(res):
-    d = flushEventualQueue()
-    def _done(ignored):
-        return res
-    d.addCallback(_done)
-    return d
+from allmydata.test.common import SystemTestMixin, MemoryConsumer, \
+     download_to_data
 
 LARGE_DATA = """
 This is some data to publish to the virtual drive, which needs to be large
@@ -50,204 +46,23 @@ class CountingDataUploadable(upload.Data):
                 self.interrupt_after_d.callback(self)
         return upload.Data.read(self, length)
 
+class GrabEverythingConsumer:
+    implements(IConsumer)
 
-class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
-                 unittest.TestCase):
+    def __init__(self):
+        self.contents = ""
 
-    def setUp(self):
-        self.sparent = service.MultiService()
-        self.sparent.startService()
-    def tearDown(self):
-        log.msg("shutting down SystemTest services")
-        d = self.sparent.stopService()
-        d.addBoth(flush_but_dont_ignore)
-        return d
+    def registerProducer(self, producer, streaming):
+        assert streaming
+        assert IPushProducer.providedBy(producer)
 
-    def getdir(self, subdir):
-        return os.path.join(self.basedir, subdir)
+    def write(self, data):
+        self.contents += data
 
-    def add_service(self, s):
-        s.setServiceParent(self.sparent)
-        return s
+    def unregisterProducer(self):
+        pass
 
-    def set_up_nodes(self, NUMCLIENTS=5):
-        self.numclients = NUMCLIENTS
-        iv_dir = self.getdir("introducer")
-        if not os.path.isdir(iv_dir):
-            fileutil.make_dirs(iv_dir)
-        f = open(os.path.join(iv_dir, "webport"), "w")
-        f.write("tcp:0:interface=127.0.0.1\n")
-        f.close()
-        iv = IntroducerNode(basedir=iv_dir)
-        self.introducer = self.add_service(iv)
-        d = self.introducer.when_tub_ready()
-        d.addCallback(self._get_introducer_web)
-        d.addCallback(self._set_up_stats_gatherer)
-        d.addCallback(self._set_up_key_generator)
-        d.addCallback(self._set_up_nodes_2)
-        d.addCallback(self._grab_stats)
-        return d
-
-    def _get_introducer_web(self, res):
-        f = open(os.path.join(self.getdir("introducer"), "node.url"), "r")
-        self.introweb_url = f.read().strip()
-        f.close()
-
-    def _set_up_stats_gatherer(self, res):
-        statsdir = self.getdir("stats_gatherer")
-        fileutil.make_dirs(statsdir)
-        t = Tub()
-        self.add_service(t)
-        l = t.listenOn("tcp:0")
-        p = l.getPortnum()
-        t.setLocation("localhost:%d" % p)
-
-        self.stats_gatherer = PickleStatsGatherer(t, statsdir, False)
-        self.add_service(self.stats_gatherer)
-        self.stats_gatherer_furl = self.stats_gatherer.get_furl()
-
-    def _set_up_key_generator(self, res):
-        kgsdir = self.getdir("key_generator")
-        fileutil.make_dirs(kgsdir)
-
-        self.key_generator_svc = KeyGeneratorService(kgsdir, display_furl=False)
-        self.key_generator_svc.key_generator.pool_size = 4
-        self.key_generator_svc.key_generator.pool_refresh_delay = 60
-        self.add_service(self.key_generator_svc)
-
-        d = fireEventually()
-        def check_for_furl():
-            return os.path.exists(os.path.join(kgsdir, 'key_generator.furl'))
-        d.addCallback(lambda junk: self.poll(check_for_furl, timeout=30))
-        def get_furl(junk):
-            kgf = os.path.join(kgsdir, 'key_generator.furl')
-            self.key_generator_furl = file(kgf, 'rb').read().strip()
-        d.addCallback(get_furl)
-        return d
-
-    def _set_up_nodes_2(self, res):
-        q = self.introducer
-        self.introducer_furl = q.introducer_url
-        self.clients = []
-        basedirs = []
-        for i in range(self.numclients):
-            basedir = self.getdir("client%d" % i)
-            basedirs.append(basedir)
-            fileutil.make_dirs(basedir)
-            if i == 0:
-                # client[0] runs a webserver and a helper, no key_generator
-                open(os.path.join(basedir, "webport"), "w").write("tcp:0:interface=127.0.0.1")
-                open(os.path.join(basedir, "run_helper"), "w").write("yes\n")
-                open(os.path.join(basedir, "sizelimit"), "w").write("10GB\n")
-            if i == 3:
-                # client[3] runs a webserver and uses a helper, uses key_generator
-                open(os.path.join(basedir, "webport"), "w").write("tcp:0:interface=127.0.0.1")
-                kgf = "%s\n" % (self.key_generator_furl,)
-                open(os.path.join(basedir, "key_generator.furl"), "w").write(kgf)
-            open(os.path.join(basedir, "introducer.furl"), "w").write(self.introducer_furl)
-            open(os.path.join(basedir, "stats_gatherer.furl"), "w").write(self.stats_gatherer_furl)
-
-        # start client[0], wait for it's tub to be ready (at which point it
-        # will have registered the helper furl).
-        c = self.add_service(client.Client(basedir=basedirs[0]))
-        self.clients.append(c)
-        d = c.when_tub_ready()
-        def _ready(res):
-            f = open(os.path.join(basedirs[0],"private","helper.furl"), "r")
-            helper_furl = f.read()
-            f.close()
-            self.helper_furl = helper_furl
-            f = open(os.path.join(basedirs[3],"helper.furl"), "w")
-            f.write(helper_furl)
-            f.close()
-
-            # this starts the rest of the clients
-            for i in range(1, self.numclients):
-                c = self.add_service(client.Client(basedir=basedirs[i]))
-                self.clients.append(c)
-            log.msg("STARTING")
-            return self.wait_for_connections()
-        d.addCallback(_ready)
-        def _connected(res):
-            log.msg("CONNECTED")
-            # now find out where the web port was
-            l = self.clients[0].getServiceNamed("webish").listener
-            port = l._port.getHost().port
-            self.webish_url = "http://localhost:%d/" % port
-            # and the helper-using webport
-            l = self.clients[3].getServiceNamed("webish").listener
-            port = l._port.getHost().port
-            self.helper_webish_url = "http://localhost:%d/" % port
-        d.addCallback(_connected)
-        return d
-
-    def _grab_stats(self, res):
-        d = self.stats_gatherer.poll()
-        return d
-
-    def bounce_client(self, num):
-        c = self.clients[num]
-        d = c.disownServiceParent()
-        # I think windows requires a moment to let the connection really stop
-        # and the port number made available for re-use. TODO: examine the
-        # behavior, see if this is really the problem, see if we can do
-        # better than blindly waiting for a second.
-        d.addCallback(self.stall, 1.0)
-        def _stopped(res):
-            new_c = client.Client(basedir=self.getdir("client%d" % num))
-            self.clients[num] = new_c
-            self.add_service(new_c)
-            return new_c.when_tub_ready()
-        d.addCallback(_stopped)
-        d.addCallback(lambda res: self.wait_for_connections())
-        def _maybe_get_webport(res):
-            if num == 0:
-                # now find out where the web port was
-                l = self.clients[0].getServiceNamed("webish").listener
-                port = l._port.getHost().port
-                self.webish_url = "http://localhost:%d/" % port
-        d.addCallback(_maybe_get_webport)
-        return d
-
-    def add_extra_node(self, client_num, helper_furl=None,
-                       add_to_sparent=False):
-        # usually this node is *not* parented to our self.sparent, so we can
-        # shut it down separately from the rest, to exercise the
-        # connection-lost code
-        basedir = self.getdir("client%d" % client_num)
-        if not os.path.isdir(basedir):
-            fileutil.make_dirs(basedir)
-        open(os.path.join(basedir, "introducer.furl"), "w").write(self.introducer_furl)
-        if helper_furl:
-            f = open(os.path.join(basedir, "helper.furl") ,"w")
-            f.write(helper_furl+"\n")
-            f.close()
-
-        c = client.Client(basedir=basedir)
-        self.clients.append(c)
-        self.numclients += 1
-        if add_to_sparent:
-            c.setServiceParent(self.sparent)
-        else:
-            c.startService()
-        d = self.wait_for_connections()
-        d.addCallback(lambda res: c)
-        return d
-
-    def _check_connections(self):
-        for c in self.clients:
-            ic = c.introducer_client
-            if not ic.connected_to_introducer():
-                return False
-            if len(ic.get_all_peerids()) != self.numclients:
-                return False
-        return True
-
-    def wait_for_connections(self, ignored=None):
-        # TODO: replace this with something that takes a list of peerids and
-        # fires when they've all been heard from, instead of using a count
-        # and a threshold
-        return self.poll(self._check_connections, timeout=200)
+class SystemTest(SystemTestMixin, unittest.TestCase):
 
     def test_connections(self):
         self.basedir = "system/SystemTest/test_connections"
@@ -316,9 +131,10 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
             return d1
         d.addCallback(_do_upload)
         def _upload_done(results):
-            uri = results.uri
-            log.msg("upload finished: uri is %s" % (uri,))
-            self.uri = uri
+            theuri = results.uri
+            log.msg("upload finished: uri is %s" % (theuri,))
+            self.uri = theuri
+            assert isinstance(self.uri, str), self.uri
             dl = self.clients[1].getServiceNamed("downloader")
             self.downloader = dl
         d.addCallback(_upload_done)
@@ -364,6 +180,51 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
             self.failUnlessEqual(newdata, DATA)
         d.addCallback(_download_to_filehandle_done)
 
+        consumer = GrabEverythingConsumer()
+        ct = download.ConsumerAdapter(consumer)
+        d.addCallback(lambda res:
+                      self.downloader.download(self.uri, ct))
+        def _download_to_consumer_done(ign):
+            self.failUnlessEqual(consumer.contents, DATA)
+        d.addCallback(_download_to_consumer_done)
+
+        def _test_read(res):
+            n = self.clients[1].create_node_from_uri(self.uri)
+            d = download_to_data(n)
+            def _read_done(data):
+                self.failUnlessEqual(data, DATA)
+            d.addCallback(_read_done)
+            d.addCallback(lambda ign:
+                          n.read(MemoryConsumer(), offset=1, size=4))
+            def _read_portion_done(mc):
+                self.failUnlessEqual("".join(mc.chunks), DATA[1:1+4])
+            d.addCallback(_read_portion_done)
+            d.addCallback(lambda ign:
+                          n.read(MemoryConsumer(), offset=2, size=None))
+            def _read_tail_done(mc):
+                self.failUnlessEqual("".join(mc.chunks), DATA[2:])
+            d.addCallback(_read_tail_done)
+            d.addCallback(lambda ign:
+                          n.read(MemoryConsumer(), size=len(DATA)+1000))
+            def _read_too_much(mc):
+                self.failUnlessEqual("".join(mc.chunks), DATA)
+            d.addCallback(_read_too_much)
+
+            return d
+        d.addCallback(_test_read)
+
+        def _test_bad_read(res):
+            bad_u = uri.from_string_filenode(self.uri)
+            bad_u.key = self.flip_bit(bad_u.key)
+            bad_n = self.clients[1].create_node_from_uri(bad_u.to_string())
+            # this should cause an error during download
+
+            d = self.shouldFail2(NotEnoughSharesError, "'download bad node'",
+                                 None,
+                                 bad_n.read, MemoryConsumer(), offset=2)
+            return d
+        d.addCallback(_test_bad_read)
+
         def _download_nonexistent_uri(res):
             baduri = self.mangle_uri(self.uri)
             log.msg("about to download non-existent URI", level=log.UNUSUAL,
@@ -373,7 +234,7 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
                 log.msg("finished downloading non-existend URI",
                         level=log.UNUSUAL, facility="tahoe.tests")
                 self.failUnless(isinstance(res, Failure))
-                self.failUnless(res.check(download.NotEnoughSharesError),
+                self.failUnless(res.check(NotEnoughSharesError),
                                 "expected NotEnoughSharesError, got %s" % res)
                 # TODO: files that have zero peers should get a special kind
                 # of NotEnoughSharesError, which can be used to suggest that
@@ -390,7 +251,6 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
                                                       add_to_sparent=True))
         def _added(extra_node):
             self.extra_node = extra_node
-            extra_node.getServiceNamed("storage").sizelimit = 0
         d.addCallback(_added)
 
         HELPER_DATA = "Data that needs help to upload" * 1000
@@ -545,6 +405,28 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
             return d
         d.addCallback(_upload_resumable)
 
+        def _grab_stats(ignored):
+            # the StatsProvider doesn't normally publish a FURL:
+            # instead it passes a live reference to the StatsGatherer
+            # (if and when it connects). To exercise the remote stats
+            # interface, we manually publish client0's StatsProvider
+            # and use client1 to query it.
+            sp = self.clients[0].stats_provider
+            sp_furl = self.clients[0].tub.registerReference(sp)
+            d = self.clients[1].tub.getReference(sp_furl)
+            d.addCallback(lambda sp_rref: sp_rref.callRemote("get_stats"))
+            def _got_stats(stats):
+                #print "STATS"
+                #from pprint import pprint
+                #pprint(stats)
+                s = stats["stats"]
+                self.failUnlessEqual(s["storage_server.accepting_immutable_shares"], 1)
+                c = stats["counters"]
+                self.failUnless("storage_server.allocate" in c)
+            d.addCallback(_got_stats)
+            return d
+        d.addCallback(_grab_stats)
+
         return d
 
     def _find_shares(self, basedir):
@@ -555,13 +437,15 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
             if not filenames:
                 continue
             pieces = dirpath.split(os.sep)
-            if pieces[-4] == "storage" and pieces[-3] == "shares":
+            if (len(pieces) >= 5
+                and pieces[-4] == "storage"
+                and pieces[-3] == "shares"):
                 # we're sitting in .../storage/shares/$START/$SINDEX , and there
                 # are sharefiles here
                 assert pieces[-5].startswith("client")
                 client_num = int(pieces[-5][-1])
                 storage_index_s = pieces[-1]
-                storage_index = storage.si_a2b(storage_index_s)
+                storage_index = si_a2b(storage_index_s)
                 for sharename in filenames:
                     shnum = int(sharename)
                     filename = os.path.join(dirpath, sharename)
@@ -572,7 +456,7 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
         return shares
 
     def _corrupt_mutable_share(self, filename, which):
-        msf = storage.MutableShareFile(filename)
+        msf = MutableShareFile(filename)
         datav = msf.readv([ (0, 1000000) ])
         final_share = datav[0]
         assert len(final_share) < 1000000 # ought to be truncated
@@ -614,13 +498,14 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
                                                 enc_privkey)
         msf.writev( [(0, final_share)], None)
 
+
     def test_mutable(self):
         self.basedir = "system/SystemTest/test_mutable"
         DATA = "initial contents go here."  # 25 bytes % 3 != 0
         NEWDATA = "new contents yay"
         NEWERDATA = "this is getting old"
 
-        d = self.set_up_nodes()
+        d = self.set_up_nodes(use_key_generator=True)
 
         def _create_mutable(res):
             c = self.clients[0]
@@ -644,7 +529,7 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
             log.msg(" for clients[%d]" % client_num)
 
             out,err = StringIO(), StringIO()
-            rc = runner.runner(["dump-share",
+            rc = runner.runner(["debug", "dump-share", "--offsets",
                                 filename],
                                stdout=out, stderr=err)
             output = out.getvalue()
@@ -823,9 +708,9 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
             d1.addCallback(lambda res: dnode.set_node(u"see recursive", dnode))
             d1.addCallback(lambda res: dnode.has_child(u"see recursive"))
             d1.addCallback(lambda answer: self.failUnlessEqual(answer, True))
-            d1.addCallback(lambda res: dnode.build_manifest())
-            d1.addCallback(lambda manifest:
-                           self.failUnlessEqual(len(manifest), 1))
+            d1.addCallback(lambda res: dnode.build_manifest().when_done())
+            d1.addCallback(lambda res:
+                           self.failUnlessEqual(len(res["manifest"]), 1))
             return d1
         d.addCallback(_created_dirnode)
 
@@ -876,7 +761,7 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
     def test_vdrive(self):
         self.basedir = "system/SystemTest/test_vdrive"
         self.data = LARGE_DATA
-        d = self.set_up_nodes()
+        d = self.set_up_nodes(use_stats_gatherer=True)
         d.addCallback(self._test_introweb)
         d.addCallback(self.log, "starting publish")
         d.addCallback(self._do_publish1)
@@ -914,7 +799,6 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
         # P/s2-rw/
         # P/test_put/  (empty)
         d.addCallback(self._test_checker)
-        d.addCallback(self._grab_stats)
         return d
     test_vdrive.timeout = 1100
 
@@ -922,7 +806,7 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
         d = getPage(self.introweb_url, method="GET", followRedirect=True)
         def _check(res):
             try:
-                self.failUnless("allmydata: %s" % str(allmydata.__version__)
+                self.failUnless("allmydata-tahoe: %s" % str(allmydata.__version__)
                                 in res)
                 self.failUnless("Announcement Summary: storage: 5, stub_client: 5" in res)
                 self.failUnless("Subscription Summary: storage: 5" in res)
@@ -942,6 +826,8 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
                                      {"storage": 5})
                 self.failUnlessEqual(data["announcement_summary"],
                                      {"storage": 5, "stub_client": 5})
+                self.failUnlessEqual(data["announcement_distinct_hosts"],
+                                     {"storage": 1, "stub_client": 1})
             except unittest.FailTest:
                 print
                 print "GET %s?t=json output was:" % self.introweb_url
@@ -965,6 +851,7 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
             d1.addCallback(self.log, "publish finished")
             def _stash_uri(filenode):
                 self.uri = filenode.get_uri()
+                assert isinstance(self.uri, str), (self.uri, filenode)
             d1.addCallback(_stash_uri)
             return d1
         d.addCallback(_made_subdir1)
@@ -976,9 +863,9 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
         d.addCallback(lambda subdir2: subdir2.add_file(u"mydata992", ut))
         return d
 
-    def log(self, res, msg, **kwargs):
-        # print "MSG: %s  RES: %s" % (msg, res)
-        log.msg(msg, **kwargs)
+    def log(self, res, *args, **kwargs):
+        # print "MSG: %s  RES: %s" % (msg, args)
+        log.msg(*args, **kwargs)
         return res
 
     def _do_publish_private(self, res):
@@ -1078,7 +965,7 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
 
             d1.addCallback(lambda res: self.shouldFail2(NotMutableError, "set_uri(nope)", None, dirnode.set_uri, u"hopeless", self.uri))
 
-            d1.addCallback(lambda res: self.shouldFail2(KeyError, "get(missing)", "'missing'", dirnode.get, u"missing"))
+            d1.addCallback(lambda res: self.shouldFail2(NoSuchChildError, "get(missing)", "missing", dirnode.get, u"missing"))
 
             personal = self._personal_node
             d1.addCallback(lambda res: self.shouldFail2(NotMutableError, "mv from readonly", None, dirnode.move_child_to, u"mydata992", personal, u"nope"))
@@ -1105,16 +992,17 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
             d1.addCallback(lambda res:
                            home.move_child_to(u"sekrit data", personal))
 
-            d1.addCallback(lambda res: home.build_manifest())
+            d1.addCallback(lambda res: home.build_manifest().when_done())
             d1.addCallback(self.log, "manifest")
-            #  four items:
+            #  five items:
+            # P/
             # P/personal/
             # P/personal/sekrit data
             # P/s2-rw  (same as P/s2-ro)
             # P/s2-rw/mydata992 (same as P/s2-rw/mydata992)
-            d1.addCallback(lambda manifest:
-                           self.failUnlessEqual(len(manifest), 4))
-            d1.addCallback(lambda res: home.deep_stats())
+            d1.addCallback(lambda res:
+                           self.failUnlessEqual(len(res["manifest"]), 5))
+            d1.addCallback(lambda res: home.start_deep_stats().when_done())
             def _check_stats(stats):
                 expected = {"count-immutable-files": 1,
                             "count-mutable-files": 0,
@@ -1314,11 +1202,11 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
             for us in self.clients[0].list_all_upload_statuses():
                 if us.get_size() > 200:
                     self._up_status = us.get_counter()
-            rs = self.clients[0].list_all_retrieve_statuses()[0]
+            rs = list(self.clients[0].list_all_retrieve_statuses())[0]
             self._retrieve_status = rs.get_counter()
-            ps = self.clients[0].list_all_publish_statuses()[0]
+            ps = list(self.clients[0].list_all_publish_statuses())[0]
             self._publish_status = ps.get_counter()
-            us = self.clients[0].list_all_mapupdate_statuses()[0]
+            us = list(self.clients[0].list_all_mapupdate_statuses())[0]
             self._update_status = us.get_counter()
 
             # and that there are some upload- and download- status pages
@@ -1425,7 +1313,9 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
             if not filenames:
                 continue
             pieces = dirpath.split(os.sep)
-            if pieces[-4] == "storage" and pieces[-3] == "shares":
+            if (len(pieces) >= 4
+                and pieces[-4] == "storage"
+                and pieces[-3] == "shares"):
                 # we're sitting in .../storage/shares/$START/$SINDEX , and there
                 # are sharefiles here
                 filename = os.path.join(dirpath, filenames[0])
@@ -1439,7 +1329,7 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
         log.msg("test_system.SystemTest._test_runner using %s" % filename)
 
         out,err = StringIO(), StringIO()
-        rc = runner.runner(["dump-share",
+        rc = runner.runner(["debug", "dump-share", "--offsets",
                             filename],
                            stdout=out, stderr=err)
         output = out.getvalue()
@@ -1469,7 +1359,7 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
         storagedir, storage_index_s = os.path.split(sharedir)
         out,err = StringIO(), StringIO()
         nodedirs = [self.getdir("client%d" % i) for i in range(self.numclients)]
-        cmd = ["find-shares", storage_index_s] + nodedirs
+        cmd = ["debug", "find-shares", storage_index_s] + nodedirs
         rc = runner.runner(cmd, stdout=out, stderr=err)
         self.failUnlessEqual(rc, 0)
         out.seek(0)
@@ -1479,7 +1369,7 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
         # also exercise the 'catalog-shares' tool
         out,err = StringIO(), StringIO()
         nodedirs = [self.getdir("client%d" % i) for i in range(self.numclients)]
-        cmd = ["catalog-shares"] + nodedirs
+        cmd = ["debug", "catalog-shares"] + nodedirs
         rc = runner.runner(cmd, stdout=out, stderr=err)
         self.failUnlessEqual(rc, 0)
         out.seek(0)
@@ -1543,16 +1433,17 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
         f.write(private_uri)
         f.close()
 
-        def run(ignored, verb, *args):
+        def run(ignored, verb, *args, **kwargs):
+            stdin = kwargs.get("stdin", "")
             newargs = [verb] + nodeargs + list(args)
-            return self._run_cli(newargs)
+            return self._run_cli(newargs, stdin=stdin)
 
         def _check_ls((out,err), expected_children, unexpected_children=[]):
             self.failUnlessEqual(err, "")
             for s in expected_children:
-                self.failUnless(s in out, s)
+                self.failUnless(s in out, (s,out))
             for s in unexpected_children:
-                self.failIf(s in out, s)
+                self.failIf(s in out, (s,out))
 
         def _check_ls_root((out,err)):
             self.failUnless("personal" in out)
@@ -1642,28 +1533,12 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
         d.addCallback(run, "get", "tahoe:file3")
         d.addCallback(_check_stdout_against, 3)
 
-        def _put_from_stdin(res, data, *args):
-            args = nodeargs + list(args)
-            o = cli.PutOptions()
-            o.parseOptions(args)
-            stdin = StringIO(data)
-            stdout, stderr = StringIO(), StringIO()
-            d = threads.deferToThread(cli.put, o,
-                                      stdout=stdout, stderr=stderr, stdin=stdin)
-            def _done(res):
-                return stdout.getvalue(), stderr.getvalue()
-            d.addCallback(_done)
-            return d
-
         #  tahoe put FOO
         STDIN_DATA = "This is the file to upload from stdin."
-        d.addCallback(_put_from_stdin,
-                      STDIN_DATA,
-                      "tahoe-file-stdin")
+        d.addCallback(run, "put", "-", "tahoe-file-stdin", stdin=STDIN_DATA)
         #  tahoe put tahoe:FOO
-        d.addCallback(_put_from_stdin,
-                      "Other file from stdin.",
-                      "tahoe:from-stdin")
+        d.addCallback(run, "put", "-", "tahoe:from-stdin",
+                      stdin="Other file from stdin.")
 
         d.addCallback(run, "ls")
         d.addCallback(_check_ls, ["tahoe-file0", "file2", "file3", "subdir",
@@ -1836,6 +1711,16 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
         # and copy it a second time, which ought to overwrite the same files
         d.addCallback(run, "cp", "-r", "tahoe:dir1", dn_copy)
 
+        # and again, only writing filecaps
+        dn_copy2 = os.path.join(self.basedir, "dir1-copy-capsonly")
+        d.addCallback(run, "cp", "-r", "--caps-only", "tahoe:dir1", dn_copy2)
+        def _check_capsonly((out,err)):
+            # these should all be LITs
+            x = open(os.path.join(dn_copy2, "subdir2", "rfile4")).read()
+            y = uri.from_string_filenode(x)
+            self.failUnlessEqual(y.data, "rfile4")
+        d.addCallback(_check_capsonly)
+
         # and tahoe-to-tahoe
         d.addCallback(run, "cp", "-r", "tahoe:dir1", "tahoe:dir1-copy")
         d.addCallback(run, "ls")
@@ -1866,10 +1751,11 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
 
         return d
 
-    def _run_cli(self, argv):
+    def _run_cli(self, argv, stdin=""):
         #print "CLI:", argv
         stdout, stderr = StringIO(), StringIO()
         d = threads.deferToThread(runner.runner, argv, run_by_human=False,
+                                  stdin=StringIO(stdin),
                                   stdout=stdout, stderr=stderr)
         def _done(res):
             return stdout.getvalue(), stderr.getvalue()
@@ -1880,21 +1766,21 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
         ut = upload.Data("too big to be literal" * 200, convergence=None)
         d = self._personal_node.add_file(u"big file", ut)
 
-        d.addCallback(lambda res: self._personal_node.check())
+        d.addCallback(lambda res: self._personal_node.check(Monitor()))
         def _check_dirnode_results(r):
             self.failUnless(r.is_healthy())
         d.addCallback(_check_dirnode_results)
-        d.addCallback(lambda res: self._personal_node.check(verify=True))
+        d.addCallback(lambda res: self._personal_node.check(Monitor(), verify=True))
         d.addCallback(_check_dirnode_results)
 
         d.addCallback(lambda res: self._personal_node.get(u"big file"))
         def _got_chk_filenode(n):
             self.failUnless(isinstance(n, filenode.FileNode))
-            d = n.check()
+            d = n.check(Monitor())
             def _check_filenode_results(r):
                 self.failUnless(r.is_healthy())
             d.addCallback(_check_filenode_results)
-            d.addCallback(lambda res: n.check(verify=True))
+            d.addCallback(lambda res: n.check(Monitor(), verify=True))
             d.addCallback(_check_filenode_results)
             return d
         d.addCallback(_got_chk_filenode)
@@ -1902,12 +1788,13 @@ class SystemTest(testutil.SignalMixin, testutil.PollMixin, testutil.StallMixin,
         d.addCallback(lambda res: self._personal_node.get(u"sekrit data"))
         def _got_lit_filenode(n):
             self.failUnless(isinstance(n, filenode.LiteralFileNode))
-            d = n.check()
-            def _check_filenode_results(r):
-                self.failUnless(r.is_healthy())
-            d.addCallback(_check_filenode_results)
-            d.addCallback(lambda res: n.check(verify=True))
-            d.addCallback(_check_filenode_results)
+            d = n.check(Monitor())
+            def _check_lit_filenode_results(r):
+                self.failUnlessEqual(r, None)
+            d.addCallback(_check_lit_filenode_results)
+            d.addCallback(lambda res: n.check(Monitor(), verify=True))
+            d.addCallback(_check_lit_filenode_results)
             return d
         d.addCallback(_got_lit_filenode)
         return d
+
