@@ -189,3 +189,7 @@ def ssk_readkey_data_hash(IV, readkey):
     return tagged_pair_hash(MUTABLE_DATAKEY_TAG, IV, readkey, KEYLEN)
 def ssk_storage_index_hash(readkey):
     return tagged_hash(MUTABLE_STORAGEINDEX_TAG, readkey, KEYLEN)
+
+def constant_time_compare(a, b):
+    n = os.urandom(8)
+    return bool(tagged_hash(n, a) == tagged_hash(n, b))
